/*
  USBasp - USB in-circuit programmer for Atmel AVR controllers
  V2.0 with JTAG capability

  Thomas Fischl: AVR parts <tfischl@gmx.de>
  Lukas Schrittwieser: JTAG parts

  License........: GNU GPL v2 (see Readme.txt)
  Target.........: ATMega88 at 18 MHz
  Creation Date..: 2005-02-20
  Last change....: 2009-02-16

*/



#include "jtag.h"

/* buffer to store the bytes received from the chain */
volatile unsigned char 		jtReadBackBuffer[JT_READBACK_LENGTH];
volatile unsigned short 	jtReadBackWrInd = 0;

/* internal configuratin status: */
/* 	bit7: set means jtag disabled, 
	bit6: set means a spi hardware transfer is in progress
	bit2: SPR1
	bit1: SPR0
	bit0: SPIX2
	*/
volatile unsigned char		jtStatus	= 0x80;	
volatile unsigned char		jtTapState;


inline void JtWait();
inline void JtUseHW();
inline void JtUseSW();
unsigned char JtSendBit (unsigned char tms, unsigned char tdi);
void JtShiftByte (unsigned char byte);
void JtResetTap();
void JtWaitSPIComp();


/* initializes the hardware and disables the jtag interface */
void JtInit ()
{
	JT_DIS_BUFFERS();		/* disable the jtag buffer / level shifter */
	//JT_DDR |= (1<<JT_EN);
	jtStatus	= 0x80;		/* we are not initialized, no transfer running */
	jtReadBackWrInd = 0;
}


/* moves the TAP state machine into the update dr state */
void JtUpdateDR ()
{
	if (jtStatus & 0x80)
		return;
	
	JtUseSW();	
		
	if (jtTapState == JT_TAP_STATE_UPDATE_IR)
		return;	
		
	switch (jtTapState)
	{
		case JT_TAP_STATE_EXIT1_IR:
			JtSendBit(1,0);
			goto UpDrUpIr;
		case JT_TAP_STATE_RESET:
			JtSendBit(0,0);
UpDrUpIr:	case JT_TAP_STATE_UPDATE_IR:
		case JT_TAP_STATE_RTI:
			JtSendBit(1,0);
			JtSendBit(0,0);
			JtSendBit(1,0);
		case JT_TAP_STATE_EXIT1_DR:
			JtSendBit(1,0);
			jtTapState = JT_TAP_STATE_UPDATE_DR;
			return;
		default:
			JtResetTap();
	}
	
	return;
}


/* moves the TAP state machine into the update ir state */
void JtUpdateIR ()
{
	if (jtStatus & 0x80)
		return;
	
	JtUseSW();	
		
	if (jtTapState == JT_TAP_STATE_UPDATE_IR)
		return;	
		
	switch (jtTapState)
	{
		case JT_TAP_STATE_EXIT1_DR:
			JtSendBit(1,0);
			goto UpIrUpDr;
		case JT_TAP_STATE_RESET:
			JtSendBit(0,0);
UpIrUpDr:	case JT_TAP_STATE_UPDATE_DR:
		case JT_TAP_STATE_RTI:
			JtSendBit(1,0);
			JtSendBit(1,0);
			JtSendBit(0,0);
			JtSendBit(1,0);
		case JT_TAP_STATE_EXIT1_IR:
			JtSendBit(1,0);
			jtTapState = JT_TAP_STATE_UPDATE_IR;
			return;
		default:
			JtResetTap();
	}
	return;
}


/* moves the TAP state machine into the run-test/idle state */
void JtRTI ()
{
	if (jtStatus & 0x80)
		return;
	
	JtUseSW();	
		
	switch (jtTapState)
	{
		case JT_TAP_STATE_EXIT1_DR:
		case JT_TAP_STATE_EXIT1_IR:
			JtSendBit(1,0);
		case JT_TAP_STATE_RESET:	
		case JT_TAP_STATE_UPDATE_DR:
		case JT_TAP_STATE_UPDATE_IR:
		case JT_TAP_STATE_RTI:		// this loop feature allows to wait cycles in rti state
			JtSendBit(0,0); 
			jtTapState = JT_TAP_STATE_RTI;
			return;
		default:
			JtResetTap();
	}
	return;
}

/* used for usb connection testing */
/* used for debuging, don't enable in production devices 
void JtWriteReadBack (unsigned char byte)
{
	jtReadBackBuffer[jtReadBackWrInd++] = byte;
	jtReadBackWrInd &= JT_READBACK_MASK;
}
*/

inline unsigned char JtGetTapState ()
{
	return jtTapState;
}


inline unsigned char JtGetReadBackWrInd()
{
	return jtReadBackWrInd;	
}

/* enables or disables the jtag engine 
	the lower 4 bit of cfg decodes as:
	0 - disable jtag
	1 - /2
	2 - /4
	3 - /8
	4 - /16
	5 - /32
	6 - /64
	7 - /64
	8 - /128
	orthers - disable jtag
	returns 1 when jtag is enabled, 0 when disabled 
*/
unsigned char JtConfig(unsigned char cfg)
{
	unsigned char clkDiv = cfg & 0x0F;
	
	if ((clkDiv == 0) || (clkDiv > 8))
	{
		JT_DIS_BUFFERS();
		jtStatus	= 0x80;	
		SPCR &= ~(1<<SPE);
		return 0;
	}
	
	clkDiv--;

	/* enable the jtag level converters / buffers */
	JT_SET_TCK();
	JT_SET_TMS();
	/* set the correct directions for the jtag interface */
	JT_DDR |= (1<<JT_TDI) | (1<<JT_TMS) | (1<<JT_TCK); // | (1<<JT_EN);
	JT_DDR &= ~(1<<JT_TDO);
	/* enalbe the jtag buffer / level converter */
	JT_EN_BUFFERS();
		
	if (jtStatus&0x80)
	{
		/* the interface was disabled until now, so reset the tap */
		jtStatus = clkDiv ^ 0x01;	/* the SPI2X bit is inverted, see data sheet */	
		JtResetTap();
	}
	else
		jtStatus = clkDiv ^ 0x01;	/* the SPI2X bit is inverted, see data sheet */	
	
	SPCR = (0<<SPIE) | (0<<SPE) | (0<<DORD) | (1<<MSTR)
	     | (1<<CPOL) | (1<<CPHA) | (((jtStatus>>2)&0x01)<<SPR1) | (((jtStatus>>1)&0x01)<<SPR0);
	SPSR = ((jtStatus&0x01)<<SPI2X); 	
		
	jtReadBackWrInd = 0;
	
	return 1;	
}

/* prepares the shifting of the instruction register */
void JtPreShiftIR ()
{
	//if (jtStatus & 0x80)
	//	return;
	
	JtUseSW();
	
	switch (jtTapState)
	{
		case JT_TAP_STATE_EXIT1_IR:
			JtSendBit(0,0);
			JtSendBit(1,0);
			goto ShIR2;
		case JT_TAP_STATE_RESET:
			JtSendBit(0,0); /* move to RTI */
			goto ShIR1;
		case JT_TAP_STATE_EXIT1_DR:
			JtSendBit(1,0); /* move to UPDATE DR */
		case JT_TAP_STATE_UPDATE_DR:
		case JT_TAP_STATE_UPDATE_IR:
		case JT_TAP_STATE_RTI:	
ShIR1:		JtSendBit(1,0); /* move to SELECT-DR-SCAN */
			JtSendBit(1,0); /* move to SELECT-IR-SCAN */
			JtSendBit(0,0);
ShIR2:		JtSendBit(0,0);
			jtTapState = JT_TAP_STATE_SHIFT_IR;
			break;
		default:
			JtResetTap();
	}
	JtUseHW();		
	return;	
}


void JtPreShiftDR ()
{
	//if (jtStatus & 0x80)
	//	return;
	
	JtUseSW();
	
	switch (jtTapState)
	{
		case JT_TAP_STATE_EXIT1_DR:
			JtSendBit(0,0);
			JtSendBit(1,0);
			goto ShDR2;
		case JT_TAP_STATE_RESET:
			JtSendBit(0,0); /* move to RTI */
			goto ShDR1;
		case JT_TAP_STATE_EXIT1_IR:
			JtSendBit(1,0); /* move to UPDATE DR */
		case JT_TAP_STATE_UPDATE_DR:
		case JT_TAP_STATE_UPDATE_IR:
		case JT_TAP_STATE_RTI:	
ShDR1:		JtSendBit(1,0); /* move to SELECT-DR-SCAN */
			JtSendBit(0,0);
ShDR2:		JtSendBit(0,0);
			jtTapState = JT_TAP_STATE_SHIFT_DR;
			break;
		default:
			JtResetTap();
	}
		
	JtUseHW();		
	return;			
}


/* shifts the last byte using software shifting and moves the TAP to EXIT1-XR state */
void JtShiftLastByte (unsigned char byte, unsigned char numBits)
{
	unsigned char tdo;
	
	/* wait until the last byte is done */
	JtWaitSPIComp();
	
	JtUseSW();
	for (;numBits>0;numBits--)
	{
		if (numBits > 1)
			tdo = JtSendBit(0, byte&0x80);	/* don't set TMS */
		else
			tdo = JtSendBit(1, byte&0x80);	/* last bit, set TMS */
		byte <<= 1;	
		/* assemble the receive byte */
		byte |= tdo;
		
	}
	jtReadBackBuffer[jtReadBackWrInd++] = byte;	/* store the last received answer bits */
	jtReadBackWrInd &= JT_READBACK_MASK;
	
	/* the tap is now in the according EXIT1 state */
	if (jtTapState == JT_TAP_STATE_SHIFT_IR)
		jtTapState = JT_TAP_STATE_EXIT1_IR;
	else
		jtTapState = JT_TAP_STATE_EXIT1_DR;
}


/* sends on byte using the hardware spi interface */
/* use this in SHIFT_DR or SHIFT_IR state */
/* the answer byte will be transferd at the next call to JtShiftByte or JtShiftLastByte */
void JtShiftByte (unsigned char byte)
{
	if (jtStatus&0x40) /* if bit 6 is set a transfer is running */
		JtWaitSPIComp();
	
	/* start the transfer */
	SPDR = byte;
	
	jtStatus |= 0x40;	/* the transfer is still running */
}

void JtResetTap()
{
	unsigned char i;
	JtUseSW();	/* use the software interface */
	
	/* send at least 5 pulses with TMS high */
	for (i=0; i<6; i++)
		JtSendBit(1, 1); 
	
	jtTapState = JT_TAP_STATE_RESET;
}

/* local functions */

void JtWait()
{
	unsigned char i;
	/* waits approx 2us, assuming 5 cycles loop length */
	for (i=0;i<5;i++)
		asm("nop");	/* to prevent the compiler optimizing away the loop */
}


inline void JtUseHW()
{
	JT_SET_TCK();	/* ensure the tck line is high */
	JT_CLR_TMS();	/* if we shift with the hardware spi inteface tms is always low */
	/* enalbe the spi hardware */	
	SPCR |= (1<<SPE);   
}

inline void JtUseSW()
{
	JT_SET_TCK();	/* ensure the tck line is high */

	
	/* disable the hardware spi interface */
	SPCR &= ~(1<<SPE);
}

/* sends a bit using a software spi algorithm */
unsigned char JtSendBit (unsigned char tms, unsigned char tdi)
{
	unsigned char tdo;
	
	JtWait();
	JT_CLR_TCK();	/* clear tck line, the next bit is prepared on tdo */
	
	/* prepare data */
	if (tdi)
		JT_SET_TDI();
	else
		JT_CLR_TDI();
	if (tms)
		JT_SET_TMS();
	else
		JT_CLR_TMS();
		
	JtWait();
	JtWait();	
			
	/* sample tdo pin */
	if (JT_GET_TDO())
		tdo = 1;
	else
		tdo = 0;
		
	JT_SET_TCK();	/* set tck high, the jtag device latches tdi and tms */	
	JtWait();
	return tdo;	/* return the sampled value */
}

/* wait for an spi transfer to finish and copy the answer byte */
void JtWaitSPIComp()
{
	if (!(jtStatus&0x40))
		return;
	
	/* wait for the last transfer to complete */
	while(!(SPSR&(1<<SPIF)));	

	/* store the last byte received from the jtag daisy chain */
	jtReadBackBuffer[jtReadBackWrInd++] = SPDR;
	
	jtStatus &= ~0x40;	/* the transfer finished */
	
	/* check the read back buffer index */
	//if (jtReadBackWrInd >= JT_READBACK_LENGTH)
	//	jtReadBackWrInd = 0;
	jtReadBackWrInd &= JT_READBACK_MASK;
}


