/*
  USBasp - USB in-circuit programmer for Atmel AVR controllers
  V2.0 with JTAG capability

  Thomas Fischl: AVR parts <tfischl@gmx.de>
  Lukas Schrittwieser: JTAG parts

  License........: GNU GPL v2 (see Readme.txt)
  Target.........: ATMega8 at 18 MHz
  Creation Date..: 2005-02-20
  Last change....: 2009-02-16

*/


#ifndef __JTAG_H__
#define __JTAG_H__

#include <avr/io.h>

/* hardware interface */
#define JT_DDR				DDRB
#define JT_PORT				PORTB
#define JT_IN				PINB
#define JT_TDI				PB3
#define JT_TMS				PB2
#define JT_TCK				PB5
#define JT_TDO				PB4
//#define JT_EN				PB2

/* macros for hardware access */
#define JT_SET_TMS()		(JT_PORT |= (1<<JT_TMS))
#define JT_CLR_TMS()		(JT_PORT &= ~(1<<JT_TMS))
#define JT_SET_TDI()		(JT_PORT |= (1<<JT_TDI))
#define JT_CLR_TDI()		(JT_PORT &= ~(1<<JT_TDI))
#define JT_SET_TCK()		(JT_PORT |= (1<<JT_TCK))
#define JT_CLR_TCK()		(JT_PORT &= ~(1<<JT_TCK))
#define JT_EN_BUFFERS() 	//(JT_PORT &= ~(1<<JT_EN))
#define JT_DIS_BUFFERS()	//(JT_PORT |= (1<<JT_EN))
#define JT_GET_TDO()		(JT_IN&(1<<JT_TDO))

/* the read back buffer can't be longer than 256 bytes (counter are shorts) */
#define JT_READBACK_LENGTH	128
#define JT_READBACK_MASK	0x7F

/* jtag TAP state machine encoding */
#define JT_TAP_STATE_RESET	 		0x00
#define JT_TAP_STATE_RTI			0x01
#define JT_TAP_STATE_SEL_DR			0x02
#define JT_TAP_STATE_SEL_IR			0x03
#define JT_TAP_STATE_CAP_DR			0x04
#define JT_TAP_STATE_CAP_IR			0x05
#define JT_TAP_STATE_SHIFT_DR		0x06
#define JT_TAP_STATE_SHIFT_IR		0x07
#define JT_TAP_STATE_EXIT1_DR		0x08
#define JT_TAP_STATE_EXIT1_IR		0x09
#define JT_TAP_STATE_PAUSE_DR		0x0A
#define JT_TAP_STATE_PAUSE_IR		0x0B
#define JT_TAP_STATE_EXIT2_DR		0x0C
#define JT_TAP_STATE_EXIT2_IR		0x0D
#define JT_TAP_STATE_UPDATE_DR  	0x0E
#define JT_TAP_STATE_UPDATE_IR		0x0F



/* the global read back buffer */
extern volatile unsigned char 		jtReadBackBuffer[];



/* initializes the hardware and disables the jtag interface */
void JtInit ();


/* enables or disables the jtag engine 
	clkDiv decodes as:
	0 - disable jtag
	1 - /2
	2 - /4
	3 - /8
	4 - /16
	5 - /32
	6 - /64
	7 - /64
	8 - /128
	orthers - disable jtag
	returns 1 when jtag is enabled, 0 when disabled
*/
unsigned char JtConfig(unsigned char clkDiv);


/* prepare the TAP to shift data or instruction register */
void JtPreShiftDR ();
void JtPreShiftIR ();

/* shifts one byte using the hardware interface */
/* the tap has to be in SHIFT-XR state */
void JtShiftByte (unsigned char byte);

/* shifts the last byte using software shifting and moves the TAP to EXIT1-XR state */
void JtShiftLastByte (unsigned char byte, unsigned char numBits);

/* moves the TAP state machine into the update dr state */
void JtUpdateDR ();

/* moves the TAP state machine into the update ir state */
void JtUpdateIR ();

/* moves the TAP state machine into the run-test/idle state */
void JtRTI ();

/* moves the TAP into RESET state */
void JtResetTap();

/* returns the current TAP state */
inline unsigned char JtGetTapState ();

/* returns the adress of the next byte to be written */
/* this is used to calculate the start address of an read instruction from the host */
inline unsigned char JtGetReadBackWrInd();

/* used for usb connection testing */
/* don't enable for production devices 
void JtWriteReadBack (unsigned char byte);
*/















#endif
/* end of file jtag.h */
