/*
  USBasp - USB in-circuit programmer for Atmel AVR controllers
  V2.0 with JTAG capability

  Thomas Fischl: AVR parts <tfischl@gmx.de>
  Lukas Schrittwieser: JTAG parts

  License........: GNU GPL v2 (see Readme.txt)
  Target.........: ATMega8 at 18 MHz
  Creation Date..: 2005-02-20
  Last change....: 2009-02-16

*/

/* mods for usbasp 2.0

different hardware pins for usb interface and new jtag interface
using long usb transfers - 0xff replaced by 0xFFFF in usbFunctionSetup!
  


*/


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "usbdrv.h"
#include "isp.h"
#include "clock.h"
#include "jtag.h"
#include "oddebug.h"

/* USBAVR commands (sent as USB vendor specific setup requests */
#define USBASP_FUNC_CONNECT     	1
#define USBASP_FUNC_DISCONNECT  	2
#define USBASP_FUNC_TRANSMIT    	3
#define USBASP_FUNC_READFLASH   	4
#define USBASP_FUNC_ENABLEPROG  	5
#define USBASP_FUNC_WRITEFLASH  	6
#define USBASP_FUNC_READEEPROM  	7
#define USBASP_FUNC_WRITEEEPROM 	8
#define USBASP_FUNC_SETLONGADDRESS 	9
#define USBASP_FUNC_SETISPSCK 		10
/* jtag commands */
#define USBASP_JTAG_CONF		0x80		/* connect or disconnect */
#define USBASP_JTAG_GET_STATE	0x81		/* get actual tap state */
#define USBASP_JTAG_READ_BACK	0x82		/* get values from readback buffer */
#define USBASP_JTAG_EXECUTE		0x83		/* execute a command sequence */

/* possible commands for the jtag command sequence */
#define JTAG_CM_SHIFT_DR	0x10
#define JTAG_CM_SHIFT_IR	0x20
#define JTAG_CM_RUN_TEST	0x30
#define JTAG_CM_UPDATE_DR	0x40
#define JTAG_CM_UPDATE_IR	0x50
#define JTAG_CM_RESET_TAP	0x60


/* avr isp states */
#define PROG_STATE_IDLE         0
#define PROG_STATE_WRITEFLASH   1
#define PROG_STATE_READFLASH    2
#define PROG_STATE_READEEPROM   3
#define PROG_STATE_WRITEEEPROM  4
#define PROG_STATE_TEST_CONNECTION 0x40
/* jtag states */
#define PROG_STATE_JTAG_IDLE		0x80
#define PROG_STATE_JTAG_EXEC		0x81
#define PROG_STATE_JTAG_READBACK	0x82


/* mask for jtag related states */
#define PROG_STATE_JTAG_MASK	0x80


#define PROG_BLOCKFLAG_FIRST    1
#define PROG_BLOCKFLAG_LAST     2

#define ledRedOn()    PORTC &= ~(1 << PC1)
#define ledRedOff()   PORTC |= (1 << PC1)
#define ledGreenOn()  PORTC &= ~(1 << PC0)
#define ledGreenOff() PORTC |= (1 << PC0)

static uchar replyBuffer[8];

static unsigned char jtCmd;
static uchar jtagReadBackIndex;

static uchar prog_state = PROG_STATE_IDLE;
static uchar prog_sck = USBASP_ISP_SCK_AUTO;

static uchar prog_address_newmode = 0;
static unsigned long prog_address;
static unsigned int prog_nbytes = 0;
static unsigned int prog_pagesize;
static uchar prog_blockflags;
static uchar prog_pagecounter;

// used to check the data0/1 token toggling
volatile uchar usbLastDataToken = 0xFF;


/* decodes jtag-related functions messages from the host */
usbMsgLen_t decodeJtagFunction (usbRequest_t *data)
{
	if (data->bRequest == USBASP_JTAG_CONF)
	{
		if(JtConfig((unsigned char)(data->wValue.bytes[0])))
		{
			prog_state = PROG_STATE_JTAG_IDLE;
			replyBuffer[0] = 1;
			ledRedOn();
		}
		else
		{
			prog_state = PROG_STATE_IDLE;
			replyBuffer[0] = 0;	
			ledRedOff();
		}
		usbMsgPtr = replyBuffer;
		return 1;	/* we return one data byte */
	}
	
	/* all other functions require a JTAG_CONF instruction first */
	if (!(prog_state & 0x80))
	{
		/* illegal instruction */
		return 0;	
	}
	
	if (data->bRequest == USBASP_JTAG_EXECUTE)
	{
		jtCmd = 0;	/* there is no active command right now */
		prog_nbytes = data->wLength.word;	/* the number of bytes to be processed */
		prog_state = PROG_STATE_JTAG_EXEC;	
		return 0xFFFF; /* multiple in instruction, done in usbFunctionWrite */
	}
	
	if (data->bRequest ==  USBASP_JTAG_READ_BACK)
	{
		/* calculate the start index for the read back instruction 
		   ReadBackWrInd points to the next byte to write, wLength is the number of bytes to transfer */
		jtagReadBackIndex = (unsigned char)(JtGetReadBackWrInd() - data->wLength.bytes[0]);
		/* correct the read index. If the number is to big we add the size -> move foreward until we hit the buffer */
		jtagReadBackIndex &= JT_READBACK_MASK;
		prog_state = PROG_STATE_JTAG_READBACK;
		return 0xFFFF;	/* multiple out insturction */
	}
	
	if (data->bRequest == USBASP_JTAG_GET_STATE)
	{
		replyBuffer[0] = JtGetTapState();
		usbMsgPtr = replyBuffer;
		return 1; /* return one data byte */	
	}
		
	return 0;	/* unknown instruction */
}



usbMsgLen_t usbFunctionSetup(uchar data[8]) 
{

  usbMsgLen_t len = 0;

  // clear the last data token check (the host initializes the data token sequence with 
  // the first data packet of setup transfer
  usbLastDataToken = 0xFF;
  
  if (data[1] & 0x80)
  	return decodeJtagFunction((usbRequest_t*)data);
  
  if(data[1] == USBASP_FUNC_CONNECT){

    /* set SCK speed */
	if ((PINC & (1 << PC2)) == 0) {
      ispSetSCKOption(ISP_SCK_SLOW);
    } else {
      ispSetSCKOption(prog_sck);
    }

    /* set compatibility mode of address delivering */
    prog_address_newmode = 0;

    ledRedOn();
    ispConnect();

  } else if (data[1] == USBASP_FUNC_DISCONNECT) {
    ispDisconnect();
    ledRedOff();

  } else if (data[1] == USBASP_FUNC_TRANSMIT) {
    replyBuffer[0] = ispTransmit(data[2]);
    replyBuffer[1] = ispTransmit(data[3]);
    replyBuffer[2] = ispTransmit(data[4]);
    replyBuffer[3] = ispTransmit(data[5]);
    len = 4;

  } else if (data[1] == USBASP_FUNC_READFLASH) {
    
    if (!prog_address_newmode)
      prog_address = (data[3] << 8) | data[2];
    
    prog_nbytes = (data[7] << 8) | data[6];
    prog_state = PROG_STATE_READFLASH;
    len = 0xFFFF; /*0xff;  multiple in */

  } else if (data[1] == USBASP_FUNC_READEEPROM) {
    
    if (!prog_address_newmode)
       prog_address = (data[3] << 8) | data[2];

    prog_nbytes = (data[7] << 8) | data[6];
    prog_state = PROG_STATE_READEEPROM;
    len = 0xFFFF; /* 0xff;  multiple in */

  } else if (data[1] == USBASP_FUNC_ENABLEPROG) {
    replyBuffer[0] = ispEnterProgrammingMode();;
    len = 1;

  } else if (data[1] == USBASP_FUNC_WRITEFLASH) {

    if (!prog_address_newmode)
      prog_address = (data[3] << 8) | data[2];

    prog_pagesize = data[4];
    prog_blockflags = data[5] & 0x0F;
    prog_pagesize += (((unsigned int)data[5] & 0xF0)<<4);
    if (prog_blockflags & PROG_BLOCKFLAG_FIRST) {
      prog_pagecounter = prog_pagesize;
    }
    prog_nbytes = (data[7] << 8) | data[6];
    prog_state = PROG_STATE_WRITEFLASH;
    len = 0xFFFF; /*0xff;  multiple out */

  } else if (data[1] == USBASP_FUNC_WRITEEEPROM) {

    if (!prog_address_newmode)
      prog_address = (data[3] << 8) | data[2];

    prog_pagesize = 0;
    prog_blockflags = 0;
    prog_nbytes = (data[7] << 8) | data[6];
    prog_state = PROG_STATE_WRITEEEPROM;
    len = 0xFFFF; /*0xff; multiple out */

  } else if(data[1] == USBASP_FUNC_SETLONGADDRESS) {

    /* set new mode of address delivering (ignore address delivered in commands) */
    prog_address_newmode = 1;
    /* set new address */
    prog_address = *((unsigned long*)&data[2]);
    
  } else if (data[1] == USBASP_FUNC_SETISPSCK) {

		/* set sck option */
		prog_sck = data[2];
		replyBuffer[0] = 0;
		len = 1;

	}

  usbMsgPtr = replyBuffer;

  return len;
}


uchar usbFunctionRead(uchar *data, uchar len)
{
  	uchar i;

  	/* check if the programmer is in the jtag read back state */
  	if (prog_state == PROG_STATE_JTAG_READBACK)
  	{
	  	//ledRedOff();
	  	for (i=0; i<len; i++)
		{
			data[i] = jtReadBackBuffer[jtagReadBackIndex++];	  
			if (jtagReadBackIndex >= JT_READBACK_LENGTH)
				jtagReadBackIndex = 0;
		}
		
		if (len<8)	/* if len is smaller than 8 this is the last paket */
		{
			//ledRedOn();
			prog_state = PROG_STATE_JTAG_IDLE;
		}
		
		return len;
 	}
  
  /* check if programmer is in correct read state */
  if ((prog_state != PROG_STATE_READFLASH) &&
      (prog_state != PROG_STATE_READEEPROM)) {
    return 0xff;
  }

  /* fill packet */
  for (i = 0; i < len; i++) {
    if (prog_state == PROG_STATE_READFLASH) {
      data[i] = ispReadFlash(prog_address);
    } else {
      data[i] = ispReadEEPROM(prog_address);
    }
    prog_address++;
  }

  /* last packet? */
  if (len < 8) {
    prog_state = PROG_STATE_IDLE;
  }

  return len;
}



uchar usbFunctionWrite(uchar *data, uchar len)
{
	uchar retVal = 0;
  	uchar i;
	static unsigned char byteCnt = 0;	// used for multi-byte commands (like shift)
	static unsigned char bitCnt = 0;	// 0: no command in progress, others: a shift command is running 
  	
	/* check for duplicate transfers */
	if (usbCurrentDataToken == usbLastDataToken)
		return 0;
	usbLastDataToken = usbCurrentDataToken;
	
  	/* check if we are in a JTAG execution state */
  	if (prog_state == PROG_STATE_JTAG_EXEC)
  	{
	  	for (i=0; i<len; i++) /* process all bytes */
	  	{
		 	if (jtCmd == 0)	/* if we have no pending command the next byte is a command */
		 	{
			 	ledRedOff();
			 	
			 	/* decode the command */
			 	if (((*data)&0xF0) == JTAG_CM_SHIFT_DR)
			 	{
				 	jtCmd = JTAG_CM_SHIFT_DR;
				 	bitCnt = (*data) & 0x0F;	/* the last 4 bits define the number of valid bits in the last byte */	
				 	byteCnt = 0;
			 	} else if (((*data)&0xF0) == JTAG_CM_SHIFT_IR)
			 	{
				 	jtCmd = JTAG_CM_SHIFT_IR;
				 	bitCnt = (*data) & 0x0F;	/* the last 4 bits define the number of valid bits in the last byte */	
				 	byteCnt = 0;
				} else if (*data == JTAG_CM_RUN_TEST)
			 	 	JtRTI();	
			 	else if (*data == JTAG_CM_UPDATE_DR)
			 		JtUpdateDR();
			 	else if (*data == JTAG_CM_UPDATE_IR)
			 		JtUpdateIR();
				else if (*data == JTAG_CM_RESET_TAP)
					JtResetTap();
				else
				{
					prog_state = PROG_STATE_JTAG_IDLE;	// illegal instruction, abort sequence
					break;	// stop the loop
				}
				//PORTC ^= 0x02;	// toggle red led to indicate transfer		
			}
		 	else
		  	{
			 	if (byteCnt == 0)	/* if byteCnt is zero this is the first byte after the command byte */
			 	{					/* it holds the number of bytes to shift */
				 	byteCnt = *data;	
				 	if (byteCnt > 0) /* if we got data to shift */	
				 	{
					 	/* prepare the TAP (move to shift state) */
					 	if (jtCmd == JTAG_CM_SHIFT_DR)
					 		JtPreShiftDR();
					 	else
					 		JtPreShiftIR();			
				 	}				 	
				}
				else 
				{
					if (byteCnt > 1)
						JtShiftByte (*data); /* this is not the last byte, shift it using the hardware spi interface */
					else 
						JtShiftLastByte (*data, bitCnt);
					byteCnt--;
				}
				if (byteCnt == 0)
					jtCmd = 0;	/* it's all done */	
		  	}
		  	data++;
		} // end of for i<len
	  	prog_nbytes -= len;
	 	if (prog_nbytes > 0)	
	 		return 0;	/* we expect more data */
  		else
  		{
	  		ledRedOn();
	  	 	prog_state = PROG_STATE_JTAG_IDLE;
	  		return 1;	/* we do not expect any more data */
  		}
	}
		
	// if we found an illegal instruction we have to discard the follwing bytes
	if (prog_state == PROG_STATE_JTAG_IDLE)
	{
		ledRedOn();
		prog_nbytes -= len;
	 	if (prog_nbytes > 0)	
	 		return 0;	/* we expect more data */
  		else
  			return 1;	/* we do not expect any more data */
	}
  	
  	/* check if programmer is in correct write state */
  	if ((prog_state != PROG_STATE_WRITEFLASH) &&
    	  (prog_state != PROG_STATE_WRITEEEPROM)) {
    	return 0xff;
  	}


  	for (i = 0; i < len; i++)
  	{
	    if (prog_state == PROG_STATE_WRITEFLASH)
	    {
      		/* Flash */
			if (prog_pagesize == 0)
			{
				/* not paged */
				ispWriteFlash(prog_address, data[i], 1);
      		} else {
				/* paged */
				ispWriteFlash(prog_address, data[i], 0);
				prog_pagecounter --;
				if (prog_pagecounter == 0) {
	  				ispFlushPage(prog_address, data[i]);
	  				prog_pagecounter = prog_pagesize;
				}
      		}

    	} else {
      		/* EEPROM */
      		ispWriteEEPROM(prog_address, data[i]);
    	}

    	prog_nbytes --;

    	if (prog_nbytes == 0) {
      		prog_state = PROG_STATE_IDLE;
      		if ((prog_blockflags & PROG_BLOCKFLAG_LAST) &&
	  			(prog_pagecounter != prog_pagesize)) {

				/* last block and page flush pending, so flush it now */
				ispFlushPage(prog_address, data[i]);
      		}
	   		retVal = 1; // Need to return 1 when no more data is to be received
    	}
    	prog_address ++;
  	}

  	return retVal;
}


int main(void)
{
  uchar   i, j;

  
     
#ifdef DEBUG_LEVEL
  DDRD |= (1<<1);
  odDebugInit();
  DBG1(0x00, 0, 0);
#endif
   
  DDRC = 0x03;          /* all inputs except PC0, PC1 */
  PORTC = 0xfe;
  
  
#ifdef DEBUG_LEVEL
  DDRD |= (1<<1);
  DBG1(0x01, 0, 0);
#endif

  clockInit();          /* init timer */
  JtInit();		/* enables jtag specific hardware */
  ispSetSCKOption(ISP_SCK_FAST);
  
  usbInit();
  
  /* Force USB reenumeration */
  usbDeviceDisconnect();
  _delay_ms(50);
  usbDeviceConnect();
  
 	/* init watchdog */
	//asm("wdr");
	/* Start timed equence */
	//WDTCSR |= (1<<WDCE) | (1<<WDE);
	/* Set new prescaler(time-out) value = 128K cycles (~1 s) */
	//WDTCSR = (1<<WDE) | (1<<WDP1) | (1<<WDP2);
	  
  sei();
 
 
#ifdef DEBUG_LEVEL
  DBG1(0x02, 0, 0);
#endif
     
  	i = 0;
  	j = 0;
  	for(;;)	        /* main event loop */
  	{
  		asm("wdr"); /* reset watch dog timer */
	  	usbPoll();

	}
  return 0;
}


